/* Copyright (c) 2004, Jason Wolfe  <jawolfe@uclink.berkeley.edu>

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
   associated documentation files (the "Software"), to deal in the Software without restriction, including 
   without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
   copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the 
   following conditions:

   The above copyright notice and this permission notice shall be included in all copies or substantial 
   portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
   TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
   DEALINGS IN THE SOFTWARE.

   ( This is the BSD/MIT Open Source Licence,
     http://www.opensource.org/licenses/mit-license.php )
*/

/*  DataClassifier.java
 *
 *  A simple abstract class which represents an interface between a Bootstrap.DataReader and a Distribution.
 *
 *  Implementations must define classifyData, which defines the mapping between Object data and Distribution d.
 *  Each data point should be given a weight of mul, and is added to the Distribution by calling the 
 *  Distribution.addDataPoint function.
 *  Each Object data is a chunk that is provided by a DataReader when it calls Distribution.classifyData.
 */

import java.io.*;

abstract class DataClassifier implements Serializable {

/*************************************************************************************************************
 * Abstract method -- must be overridden by implementations                                                  *
 *************************************************************************************************************/

    /* Classifies Object data, adding its data points to Distribution d with weight mul.
     * The contents of data are determined by the DataReader that is used
     */
    public abstract void classifyData(Distribution d, Object data, int mul);


    
    
/*************************************************************************************************************
 * Useful utility functions for classification                                                               *
 *************************************************************************************************************/
   
    /* Returns the first index i of the sorted array arr for which value<arr[i], or arr.length if 
     * value >= arr[arr.length-1]
     */
    public static int discretize(double [] arr, double value) {
        int i;
        for(i=0;i<arr.length&&value>=arr[i];i++);
        return i;
    }

    /* Calculates the distance between two points  
     */
    public static double dist(double x1, double y1, double z1, double x2, double y2, double z2) {
        return Math.sqrt(Math.pow((x1 - x2),2) + Math.pow((y1 - y2),2) + Math.pow((z1 - z2),2));
    }
    
}
