/* Copyright (c) 2004, Jason Wolfe  <jawolfe@uclink.berkeley.edu>

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
   associated documentation files (the "Software"), to deal in the Software without restriction, including 
   without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
   copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the 
   following conditions:

   The above copyright notice and this permission notice shall be included in all copies or substantial 
   portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
   TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
   DEALINGS IN THE SOFTWARE.

   ( This is the BSD/MIT Open Source Licence,
     http://www.opensource.org/licenses/mit-license.php )
*/

/*  tmplReader.java
 *
 *  A extention of the Bootstrap.DataReader class which reads in a SCOP data set.
 *
 *  The data set should have one .tmpl file per sequence, and all the .tmpl files should be placed in a 
 *  single folder. 
 */


import java.util.*;
import java.io.*;

public class tmplReader extends Bootstrap.DataReader {
/*************************************************************************************************************
 * Alphabets and mappings used in reading a .tmpl file                                                       *
 *************************************************************************************************************/
    
    static final String AAs = "ACDEFGHIKLMNPQRSTVWY";   // Amino acid alphabet
    
    static final String STypes = "HGIEBbTSC _-L";       // Full SS alphabet

    static final String STypes_map8  = "HGIEBBTSLLLLL"; // Mapping from full SS alphabet to 8-state alphabet
    static final String STypes_red8  = "HGIEBTSL";      // 8-state SS alphabet

    static final String STypes_mapFA  = "HLLELLLLLLLLL";// FA mapping from full SS alphabet to 3-state reduced alphabet
    static final String STypes_red3 = "HEL";            // 3-State reduced SS alphabet

    
/*************************************************************************************************************
 * Member Variable                                                                                           *
 *************************************************************************************************************/

    public String folder;  // The folder that holds the SCOP data set

/*************************************************************************************************************
 * Methods                                                                                                   *
 *************************************************************************************************************/

    /*  Constructor:  Initialized a tmplReader that reads from a data set stored in folder <fold> 
     */
    public tmplReader(String fold) {
        folder = fold;
    }
    
    
    /*  Train an array of Distributions on this SCOP data set, possibly training some Distributions on a bootstrap
     *  sample of the data set.
     *
     *  sInd gives the first index in dists to use bootstrap sampling on.  Distributions earlier in Dists are trained
     *  regularly, as with the train function above.
     */
    public void bootstrap(Distribution [] dists, int sInd) {
        File f = new File(folder);
        File [] dfiles= f.listFiles();
        int incr = dfiles.length/100;
        for(int i = 0; i < dfiles.length; i++) {
            if (dfiles[i].getName().substring(dfiles[i].getName().length()-5).equals(".tmpl")) {
                doFile(dfiles[i],dists,sInd);
                if (i % incr==0) System.out.print(".");
            }
        }
        System.out.println();
    }

  
    /* Processes a .tmpl file, calling readFile to read it
     * into a list of Residues and classifyData to process it
     * If a file has an error, it is ignored.
     * Bootstraps Distributions with index in dist >= sInd.
     */
    public static void doFile(File f, Distribution [] dist, int sInd) {
        List l = readFile(f);
        if (l != null) {
            Residue [] seq = (Residue [])l.toArray(new Residue[l.size()]);
            for(int i = 0; i < dist.length;i++) {
                if (i >= sInd) {
                    int p = (int)random.nextPoisson();
                    if (p > 0) dist[i].classifyData(seq,p);
                } else dist[i].classifyData(seq,1);
            }
        } else {
            System.err.print("Error in input file: " + f.getName());
        }
    }
    
    
    
    /* Read in a .tmpl file as a List of Residues
     * If a residue has any missing information (including residue identity or SS of 'X'), it is thrown out and 
     * replaced by a 'null' placeholder in the List.
     * */
    public static List readFile(File inf) {
        FileInputStream fs = null;
        try {
            fs = new FileInputStream(inf);
            while(fs.read() != '\n');
        } catch (Exception e) {
            System.err.print("File not found\n");
            System.exit(1);
        }
        Reader rd = new BufferedReader(new InputStreamReader(fs));
        
        StreamTokenizer st = new StreamTokenizer(rd);
        st.parseNumbers();
        st.eolIsSignificant(true);
        st.ordinaryChars(',',',');
        st.whitespaceChars(' ',' ');
        st.ordinaryChars('A','Z');
        st.ordinaryChars('a','z');
        List Residues = new LinkedList();
        int seqnum=0;
        try {
            while(st.nextToken() != StreamTokenizer.TT_EOF) {
                Residue r = readRes(st);
                if (r != null) r.sLoc=seqnum;
                seqnum++;
                Residues.add(r);
                while(st.ttype != StreamTokenizer.TT_EOL) st.nextToken();
            }
        } catch (Exception e) {
            System.err.print("Error tokenizing stream\n");
            System.exit(1);
        }
        for(Iterator i = Residues.iterator(); i.hasNext() ;) {
            Residue r = (Residue)i.next();
            if (r != null) {
                r.sLen = Residues.size();
            }
        }
        return Residues;
    }
    
    
    /* Attempt to read an individual Residue from the file
     * represented by st.  Returns null if there is a
     * formatting error, or any information is missing
     */
    public static Residue readRes(StreamTokenizer st) throws IOException {
        Residue r = new Residue();
        if (st.ttype != ',') st.nextToken();
        st.nextToken();
        
        st.nextToken();
        if (st.ttype != ',') st.nextToken();
        st.nextToken();
        
        st.nextToken();
        st.nextToken();
        
        if (AAs.indexOf(st.ttype)==-1) {
            return null;
        }
        r.aa = AAs.indexOf(st.ttype);
        st.nextToken();
        st.nextToken();
        if (STypes.indexOf(st.ttype)==-1) {
            return null;
        }
        r.stype_3 = STypes_red3.indexOf(STypes_mapFA.charAt(STypes.indexOf(st.ttype)));
        r.stype_8 = STypes_red8.indexOf(STypes_map8.charAt(STypes.indexOf(st.ttype)));
        st.nextToken();
        st.nextToken();
        if (st.ttype != StreamTokenizer.TT_NUMBER) {
            return null;
        }
        
        r.x = st.nval;
        st.nextToken();
        st.nextToken();
        if (st.ttype != StreamTokenizer.TT_NUMBER){
            return null;
        }
        
        r.y = st.nval;
        st.nextToken();
        st.nextToken();
        if (st.ttype != StreamTokenizer.TT_NUMBER) {
           return null;
        }
        
        r.z = st.nval;
        st.nextToken();
        st.nextToken();
        if (st.ttype != StreamTokenizer.TT_NUMBER) {
            return null;
        }
        
        
        r.burial = st.nval;
        
        return r;
    }
}
